@@ -356,6 +356,8 @@ class CEA_Obj(object):
         #: ERphi = Equivalence ratios in terms of fuel-to-oxidant weight ratios.
         #: ERr = Chemical equivalence ratios in terms of valences.
         #: pc_units = 'psia', 'bar', 'atm', 'mmh'(mm of mercury)
+        #: frozen flag (0=equilibrium, 1=frozen)
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         '''
 
         global _last_called, _NLines_Max_ever
@@ -528,6 +530,16 @@ class CEA_Obj(object):
             py_cea.trpts.cpfro[i] = 0.0  # frozen specific heat
             py_cea.trpts.confro[i] = 0.0 # frozen thermal conductivity
             py_cea.trpts.prfro[i] = 0.0  # frozen prandtl number
+        
+        # set species concentrations to 0.0 prior to calcs
+        for k,p in enumerate(py_cea.cdata.prod):
+            for i in range(3):
+                py_cea.comp.en[k-1,i] = 0.0
+                py_cea.therm.mw[k-1] = 0.0
+            if k>50:
+                break
+            
+            
         #print( 'calling py_cea with pathPrefix and myfile=' )
         #print( '"'+self.pathPrefix+'"',' and ', '"'+myfile+'"' )
         py_cea.py_cea(self.pathPrefix, myfile, self.makeOutput, readData)
@@ -538,6 +550,8 @@ class CEA_Obj(object):
         """
         Get the full output file created by CEA. Return as a string.
         #: pc_units = 'psia', 'bar', 'atm', 'mmh'(mm of mercury)
+        #: frozen flag (0=equilibrium, 1=frozen)
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         """
 
         # regardless of how run was set up, change makeOutput flag True
@@ -612,6 +626,7 @@ class CEA_Obj(object):
 
         #: Return the tuple (IspFrozen, Cstar, Tcomb).
         #: MR is only used for ox/fuel combos.
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         """
 
         self.setupCards( Pc=Pc, MR=MR, eps=eps, frozen=1, frozenAtThroat=frozenAtThroat)
@@ -809,6 +824,8 @@ class CEA_Obj(object):
         #: Return a list of temperatures at the chamber, throat and exit.
         #: (Note frozen flag determins whether Texit is equilibrium or Frozen temperature)
         #: MR is only used for ox/fuel combos.
+        #: frozen flag (0=equilibrium, 1=frozen)
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         """
         #self.setupCards( Pc=Pc, MR=MR, eps=eps)
         self.setupCards( Pc=Pc, MR=MR, eps=eps, frozen=frozen, frozenAtThroat=frozenAtThroat)
@@ -842,20 +859,101 @@ class CEA_Obj(object):
     def get_Enthalpies(self, Pc=100.0, MR=1.0,eps=40.0):
         """::
 
-        #: Return a list of enthalpies at the chamber, throat and exit.
+        #: Return a list of enthalpies at the chamber, throat and exit in BTU/lbm
         #: MR is only used for ox/fuel combos.
         """
         self.setupCards( Pc=Pc, MR=MR, eps=eps)
-        # convert from m/sec into ft/sec
+        
         hList =  py_cea.prtout.hsum[:3]
         for i,h in enumerate( hList ):
             hList[i] = h * 1.8 * 8314.51 / 4184.0  # convert into BTU/lbm
         return hList
 
+    def get_SpeciesMassFractions(self, Pc=100.0, MR=1.0,eps=40.0, frozen=0, frozenAtThroat=0):
+        """::
+
+        #: Returns species mass fractions at the chamber, throat and exit.
+        #: MR is only used for ox/fuel combos.
+        #: frozen flag (0=equilibrium, 1=frozen)
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
+        #: Returns 2 dictionaries
+        #: molWtD dictionary: index=species: value=molecular weight
+        #: massFracD dictionary: index=species: value=[massfrac_chm, massfrac_tht, massfrac_exit]
+        """
+        self.setupCards( Pc=Pc, MR=MR, eps=eps, frozen=frozen, frozenAtThroat=frozenAtThroat)
+        
+        massFracD = {} # index=species: value=[massfrac_chm, massfrac_tht, massfrac_exit]
+        molWtD    = {} # index=species: value=molecular weight
+        for k,p in enumerate(py_cea.cdata.prod):
+            p = p.decode("utf-8").strip()
+            if p:
+                sL = []
+                mfL = []
+                gt_zero = False
+                for i in range(3):
+                    en = py_cea.comp.en[k-1,i]
+                    mw = py_cea.therm.mw[k-1]
+                    
+                    mfL.append( en*mw )
+                    if mfL[-1] >= 0.000005:
+                        gt_zero = True
+                
+                if gt_zero:
+                    if frozen:
+                        if frozenAtThroat:
+                            mfL = [ mfL[0], mfL[1], mfL[1] ]
+                        else:
+                            mfL = [ mfL[0], mfL[0], mfL[0] ]
+                            
+                    massFracD[p] = mfL
+                    molWtD[p] = mw
+        return molWtD, massFracD
+
+    def get_SpeciesMoleFractions(self, Pc=100.0, MR=1.0,eps=40.0, frozen=0, frozenAtThroat=0):
+        """::
+
+        #: Returns species mole fractions at the chamber, throat and exit.
+        #: MR is only used for ox/fuel combos.
+        #: frozen flag (0=equilibrium, 1=frozen)
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
+        #: Returns 2 dictionaries
+        #: molWtD dictionary: index=species: value=molecular weight
+        #: moleFracD dictionary: index=species: value=[molefrac_chm, molefrac_tht, molefrac_exit]
+        """
+        self.setupCards( Pc=Pc, MR=MR, eps=eps, frozen=frozen, frozenAtThroat=frozenAtThroat)
+        
+        moleFracD = {} # index=species: value=[molefrac_chm, molefrac_tht, molefrac_exit]
+        molWtD    = {} # index=species: value=molecular weight
+        for k,p in enumerate(py_cea.cdata.prod):
+            p = p.decode("utf-8").strip()
+            if p:
+                sL = []
+                mfL = []
+                gt_zero = False
+                for i in range(3):
+                    en = py_cea.comp.en[k-1,i]
+                    totn = py_cea.prtout.totn[i]
+                    mw = py_cea.therm.mw[k-1]
+                    
+                    mfL.append( en/totn )
+                    if mfL[-1] >= 0.000005:
+                        gt_zero = True
+                
+                if gt_zero:
+                    if frozen:
+                        if frozenAtThroat:
+                            mfL = [ mfL[0], mfL[1], mfL[1] ]
+                        else:
+                            mfL = [ mfL[0], mfL[0], mfL[0] ]
+                    
+                    moleFracD[p] = mfL
+                    molWtD[p] = mw
+        return molWtD, moleFracD
+
     def get_Chamber_H(self, Pc=100.0, MR=1.0, eps=40.0):
         """::
 
-        #: Return the enthalpy in the chamber.
+        #: Return the enthalpy in the chamber in BTU/lbm
         #: MR is only used for ox/fuel combos.
         """
         hList = self.get_Enthalpies( Pc=Pc, MR=MR, eps=eps)
@@ -904,6 +1002,7 @@ class CEA_Obj(object):
 
         #: Return the heat capacity in the chamber.
         #: MR is only used for ox/fuel combos.
+        #: frozen flag (0=equilibrium, 1=frozen)
         """
         cpList = self.get_HeatCapacities( Pc=Pc, MR=MR, eps=eps, frozen=frozen)
         return cpList[0] # BTU/lbm degR
@@ -1085,6 +1184,7 @@ class CEA_Obj(object):
         #: Return the Thrust Coefficient (CF) for frozen chemistry and ambient pressure
         #: Pamb ambient pressure (e.g. sea level=14.7 psia)
         #: MR is only used for ox/fuel combos.
+        #: frozenAtThroat flag, 0=frozen in chamber, 1=frozen at throat
         """
 
         IspAmb, mode = self.estimate_Ambient_Isp( Pc=Pc, MR=MR, eps=eps, Pamb=Pamb,
