@@ -36,6 +36,6 @@ $(window).resize(function(){
         $("li.nav-item-0 a").text("Top");
     }
     else {
-        $("li.nav-item-0 a").text("RocketCEA 1.1.4 documentation");
+        $("li.nav-item-0 a").text("RocketCEA v1.1.7 documentation");
     }
 });
\ No newline at end of file
